# Generated by roxygen2: do not edit by hand

S3method(plot,timeSlice.ML)
S3method(plot,traitMedusa.model)
S3method(summary,traitMedusa)
export(ML.RatePhylo)
export(RatePhylo.allCI)
export(addFossilToPhy)
export(as.rateData)
export(as.rateMatrix)
export(calcCutOff)
export(chr.disp.lrt)
export(chr.disp.param)
export(chr.disp.sim)
export(contemporaryPhy)
export(dropTipPartial)
export(fairProportions)
export(likRatePhylo)
export(likTraitPhylo)
export(mcmc.plot)
export(node.descendents)
export(nodeTimes)
export(optim.likRatePhylo)
export(phyloCovar)
export(phyloMean)
export(phyloVar)
export(pic.pgls)
export(plot.timeSlice.ML)
export(plot.traitMedusa.model)
export(sampleShid)
export(sortTraitData)
export(summary.traitMedusa)
export(traitData.plot)
export(transformPhylo)
export(transformPhylo.MCMC)
export(transformPhylo.ML)
export(transformPhylo.ll)
export(transformPhylo.sim)
export(transformRateMatrix)
import(ape)
import(caper)
import(coda)
import(grDevices)
import(graphics)
import(ks)
import(methods)
import(mvtnorm)
import(stats)
import(utils)
importFrom(Rcpp,sourceCpp)
useDynLib(motmot)
useDynLib(motmot, .registration=TRUE)
