#' Anolis phenotype data
#'
#' Data on anolis phenotype data from Thomas et al. 2009
#'
#' @docType data
#'
#' @usage data(anolis.data)
#'
#' @format An object of class \code{"data.frame"}.
#'
#' @keywords datasets
#'
#' @references Thomas GH, Meiri S, & Phillimore AB. 2009. Body size diversification in Anolis: novel environments and island effects. Evolution 63, 2017-2030.
#' @examples
#' data(anolis.data)
#' head(anolis.data)
"anolis.data"