#' Models Of Trait Macroevolution On Trees
#'
#' Functions for fitting models of trait evolution on phylogenies for continuous
#' traits. The majority of functions described in Thomas and Freckleton (2012)
#' and include functions that allow for tests of variation in the rates of trait
#' evolution.
#'
#' @name motmot-package
#' @aliases motmot
#'
#' @docType package
#'
#' @author Mark Puttick <marknputtick@gmail.com>.
#' @author Gavin Thomas.
#' @author Travis Ingram.
#' @author Magnus Clarke.
#' @author Rob Freckleton.
#' @author David Orme.
#' @author Emmanuel Paradis.
#'
#' @keywords package
#'
#' @references
#' Thomas G. H., & Freckleton R. 2012.
#'  MOTMOT: models of trait macroevolution on trees.
#'  _Methods in Ecology and Evolution_ 3, 145-151.
#'  \doi{10.1111/j.2041-210X.2011.00132.x}
#'
#' @importFrom Rcpp sourceCpp
#' @useDynLib motmot, .registration=TRUE
"_PACKAGE"
