#' Finch phenotype data
#'
#' Finch data from Clarke et al. 2017
#'
#' @docType data
#'
#' @usage data(finches)
#'
#' @format An object of class \code{"data.frame"}.
#'
#' @keywords datasets
#'
#' @references Clarke M, Thomas GH, Freckleton RP. 2017. Trait evolution in adaptive radiations: modelling and measuring interspecific competition on phylogenies. The American Naturalist. 189, 121-137.
#' @examples
#' data(finches)
#' head(finch.data)
"finch.data"
